@@ -33,7 +33,8 @@ import os
 import subprocess
 import sys
 import tempfile
-
+import thread
+import threading
 
 # pygtk imports
 import pygtk
@@ -42,17 +43,19 @@ from gtk import gdk
 import gtk.glade
 import gobject
 
+
 # keepnote imports
 import keepnote
 from keepnote import log_error
 import keepnote.gui.richtext.richtext_tags
 from keepnote import get_resource, ensure_unicode, get_platform, unicode_gtk
+from keepnote import tasklib
 from keepnote.notebook import \
      NoteBookError
 import keepnote.notebook as notebooklib
 import keepnote.gui.dialog_app_options
 import keepnote.gui.dialog_node_icon
-import keepnote.gui
+import keepnote.gui.dialog_wait
 from keepnote.gui.icons import \
     DEFAULT_QUICK_PICK_ICONS
     
@@ -60,44 +63,67 @@ _ = keepnote.translate
 
 
 
-
 # constants
 MAX_RECENT_NOTEBOOKS = 20
-ACCEL_FILE = "accel.txt"
+ACCEL_FILE = u"accel.txt"
 CONTEXT_MENU_ACCEL_PATH = "<main>/context_menu"
 
-
-# globals
-_g_pixbufs = {}
+# font constants
+DEFAULT_FONT_FAMILY = "Sans"
+DEFAULT_FONT_SIZE = 10
+DEFAULT_FONT = "%s %d" % (DEFAULT_FONT_FAMILY, DEFAULT_FONT_SIZE)
 
 
 
 #=============================================================================
 # resources
 
-def get_pixbuf(filename, size=None):
-    """
-    Get pixbuf from a filename
+class PixbufCache (object):
+    """A cache for loading pixbufs from the filesystem"""
 
-    Cache pixbuf for later use
-    """
+    def __init__(self):
+        self._pixbufs = {}
 
-    key = (filename, size)
+
+    def get_pixbuf(self, filename, size=None, key=None):
+        """
+        Get pixbuf from a filename
+        Cache pixbuf for later use
+        """
+
+        if key is None:
+            key = (filename, size)
+
+        if key in self._pixbufs:
+            return self._pixbufs[key]
+        else:
+            # may raise GError
+            pixbuf = gtk.gdk.pixbuf_new_from_file(filename)
+
+            # resize
+            if size:
+                if size != (pixbuf.get_width(), pixbuf.get_height()):
+                    pixbuf = pixbuf.scale_simple(size[0], size[1],
+                                                 gtk.gdk.INTERP_BILINEAR)
+
+            self._pixbufs[key] = pixbuf
+            return pixbuf
+
+    def cache_pixbuf(self, pixbuf, key):
+        self._pixbufs[key] = pixbuf
+
+
+    def is_pixbuf_cached(self, key):
+        return key in self._pixbufs
     
-    if key in _g_pixbufs:
-        return _g_pixbufs[key]
-    else:
-        # may raise GError
-        pixbuf = gtk.gdk.pixbuf_new_from_file(filename)
+# singleton
+pixbufs = PixbufCache()
+
+
+get_pixbuf = pixbufs.get_pixbuf
+cache_pixbuf = pixbufs.cache_pixbuf
+is_pixbuf_cached = pixbufs.is_pixbuf_cached
 
-        if size:
-            if size != (pixbuf.get_width(), pixbuf.get_height()):
-                pixbuf = pixbuf.scale_simple(size[0], size[1],
-                                             gtk.gdk.INTERP_BILINEAR)
-        
-        _g_pixbufs[key] = pixbuf
-        return pixbuf
-    
 
 def get_resource_image(*path_list):
     """Returns gtk.Image from resource path"""
@@ -107,10 +133,24 @@ def get_resource_image(*path_list):
     img.set_from_file(filename)
     return img
 
-def get_resource_pixbuf(*path_list):
+def get_resource_pixbuf(*path_list, **options):
     """Returns cached pixbuf from resource path"""
     # raises GError
-    return get_pixbuf(get_resource(keepnote.IMAGE_DIR, *path_list))
+    return pixbufs.get_pixbuf(get_resource(keepnote.IMAGE_DIR, *path_list),
+                              **options)
+
+
+def fade_pixbuf(pixbuf, alpha=128):
+    """Returns a new faded a pixbuf"""
+    width, height = pixbuf.get_width(), pixbuf.get_height()
+    pixbuf2 = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, True, 8, width, height)
+    pixbuf2.fill(0xffffff00) # fill with transparent
+    pixbuf.composite(pixbuf2, 0, 0, width, height, 
+                     0, 0, 1.0, 1.0, gtk.gdk.INTERP_NEAREST, alpha)
+    #pixbuf.composite_color(pixbuf2, 0, 0, width, height, 
+    #                       0, 0, 1.0, 1.0, gtk.gdk.INTERP_NEAREST, alpha,
+    #                       0, 0, 1, 0xcccccc, 0x00000000)
+    return pixbuf2
 
 
 
@@ -133,6 +173,24 @@ def init_key_shortcuts():
         gtk.accel_map_save(accel_file)
 
 
+def set_gtk_style(font_size=10, vsep=0):
+    """
+    Set basic GTK style settings
+    """
+
+    gtk.rc_parse_string("""
+      style "keepnote-treeview" {
+        font_name = "%(font_size)d"
+        GtkTreeView::vertical-separator = %(vsep)d
+        GtkTreeView::expander-size = 10
+      }
+
+      class "GtkTreeView" style "keepnote-treeview"
+      class "GtkEntry" style "keepnote-treeview"
+
+      """ % {"font_size": font_size,
+             "vsep": vsep})
+
 
 def update_file_preview(file_chooser, preview):
     """Preview widget for file choosers"""
@@ -163,8 +221,8 @@ class FileChooserDialog (gtk.FileChooserDialog):
         self._persistent_path = persistent_path
         
         if self._app and self._persistent_path:
-            path = getattr(self._app.pref, self._persistent_path)
-            if os.path.exists(path):
+            path = self._app.get_default_path(self._persistent_path)
+            if path and os.path.exists(path):
                 self.set_current_folder(path)
 
 
@@ -173,8 +231,8 @@ class FileChooserDialog (gtk.FileChooserDialog):
 
         if (response == gtk.RESPONSE_OK and 
             self._app and self._persistent_path):
-            setattr(self._app.pref, self._persistent_path,
-                    unicode_gtk(self.get_current_folder()))
+            self._app.set_default_path(
+                self._persistent_path, unicode_gtk(self.get_current_folder()))
             
         return response
 
@@ -294,23 +352,34 @@ def add_actions(actiongroup, actions):
 # Application for GUI
 
 
+# TODO: implement 'close all' for notebook
+# requires listening for close.
+
+
 class KeepNote (keepnote.KeepNote):
     """GUI version of the KeepNote application instance"""
 
     def __init__(self, basedir=None):
         keepnote.KeepNote.__init__(self, basedir)
         
+        # window management
         self._current_window = None
         self._windows = []
 
+        # shared gui resources
+        self._tag_table = keepnote.gui.richtext.richtext_tags.RichTextTagTable()
+        self.app_options_dialog = keepnote.gui.dialog_app_options.ApplicationOptionsDialog(self)
+        self.node_icon_dialog = keepnote.gui.dialog_node_icon.NodeIconDialog(self)
+
+        # auto save
+        self._auto_saving = False          # True if autosave is on
+        self._auto_save_registered = False # True if autosave is registered
+        self._auto_save_pause = 0          # >0 if autosave is paused
+
 
     def init(self):
         """Initialize application from disk"""
-        keepnote.KeepNote.init(self)
-
-        self.app_options_dialog = keepnote.gui.dialog_app_options.ApplicationOptionsDialog(self)
-        self.node_icon_dialog = keepnote.gui.dialog_node_icon.NodeIconDialog(self)
-        self._tag_table = keepnote.gui.richtext.richtext_tags.RichTextTagTable()
+        keepnote.KeepNote.init(self)        
 
         
     def set_lang(self):
@@ -324,6 +393,40 @@ class KeepNote (keepnote.KeepNote):
         gtk.glade.textdomain(keepnote.GETTEXT_DOMAIN)
 
 
+    def load_preferences(self):
+        """Load information from preferences"""
+
+        keepnote.KeepNote.load_preferences(self)
+
+        # set defaults for auto save
+        p = self.pref
+        use_autosave = p.get("autosave", default=True)
+        p.get("autosave_time", default=keepnote.DEFAULT_AUTOSAVE_TIME)
+        
+
+        # set style
+        set_gtk_style(font_size=p.get("look_and_feel", "app_font_size", 
+                                      default=10))
+
+        # let windows load their preferences
+        for window in self._windows:
+            window.load_preferences()
+        
+        
+        # start autosave loop, if requested
+        self.begin_auto_save()
+
+
+    def save_preferences(self):
+        """Save information into preferences"""
+
+        # let windows save their preferences
+        for window in self._windows:
+            window.save_preferences()
+
+        keepnote.KeepNote.save_preferences(self)
+
+
     #=================================
     # GUI
 
@@ -345,6 +448,9 @@ class KeepNote (keepnote.KeepNote):
         self.init_extensions_windows([window])
         window.show_all()
 
+        if self._current_window is None:
+            self._current_window = window
+        
         return window
 
     
@@ -358,46 +464,95 @@ class KeepNote (keepnote.KeepNote):
         return self._windows
 
 
-    def open_notebook(self, filename, window=None):
+    def open_notebook(self, filename, window=None, task=None):
         """Open notebook"""
 
         from keepnote.gui import dialog_update_notebook
+        
+        # HACK
+        if isinstance(self._conns.get(filename), 
+                      keepnote.notebook.connection.fs.NoteBookConnectionFS):
 
-        version = notebooklib.get_notebook_version(filename)
-            
-        if version < notebooklib.NOTEBOOK_FORMAT_VERSION:
-            dialog = dialog_update_notebook.UpdateNoteBookDialog(self, window)
-            if not dialog.show(filename, version=version):
-                self.error(_("Cannot open notebook (version too old)"))
+            try:
+                version = notebooklib.get_notebook_version(filename)
+            except Exception, e:
+                self.error(_("Could not load notebook '%s'.") % filename,
+                           e, sys.exc_info()[2])
                 return None
 
-        # try to open notebook
-        try:
+            
+            if version < notebooklib.NOTEBOOK_FORMAT_VERSION:
+                dialog = dialog_update_notebook.UpdateNoteBookDialog(
+                    self, window)
+                if not dialog.show(filename, version=version, task=task):
+                    self.error(_("Cannot open notebook (version too old)"))
+                    gtk.gdk.threads_leave()
+                    return None
+        
+        
+        # load notebook in background
+        def update(task):
+            sem = threading.Semaphore()
+            sem.acquire()
+            
+            # perform notebook load in gui thread.
+            # Ideally, this should be in the background, but it is very
+            # slow.  If updating the wait dialog wasn't so expensive, I would
+            # simply do loading in the background thread.
+            def func():
+                conn = self._conns.get(filename)
+                notebook = notebooklib.NoteBook()
+                notebook.load(filename, conn)
+                task.set_result(notebook)
+                sem.release() # notify that notebook is loaded
+                return False
+            gobject.idle_add(func)
+
+            # wait for notebook to load
+            sem.acquire()
+            
+
+        def update_old(task):
             notebook = notebooklib.NoteBook()
             notebook.load(filename)
+            task.set_result(notebook)
 
-        except NoteBookVersionError, e:
+        task = tasklib.Task(update)
+        dialog = keepnote.gui.dialog_wait.WaitDialog(window)
+        dialog.show(_("Opening notebook"), _("Loading..."), task, cancel=False)
+        
+
+        # detect errors
+        try:
+            if task.aborted():
+                raise task.exc_info()[1]
+            else:
+                notebook = task.get_result()
+                if notebook is None:
+                    return None
+            
+
+        except notebooklib.NoteBookVersionError, e:
             self.error(_("This version of %s cannot read this notebook.\n" 
                          "The notebook has version %d.  %s can only read %d.")
                        % (keepnote.PROGRAM_NAME,
                           e.notebook_version,
                           keepnote.PROGRAM_NAME,
                           e.readable_version),
-                       e, sys.exc_info()[2])
+                       e, task.exc_info()[2])
             return None
 
         except NoteBookError, e:
             self.error(_("Could not load notebook '%s'.") % filename,
-                       e, sys.exc_info()[2])
+                       e, task.exc_info()[2])
             return None
 
         except Exception, e:
             # give up opening notebook
             self.error(_("Could not load notebook '%s'.") % filename,
-                       e, sys.exc_info()[2])
+                       e, task.exc_info()[2])
             return None
 
-
         # install default quick pick icons
         if len(notebook.pref.get_quick_pick_icons()) == 0:
             notebook.pref.set_quick_pick_icons(
@@ -407,9 +562,105 @@ class KeepNote (keepnote.KeepNote):
             # TODO: use listeners to invoke saving
             notebook.write_preferences()
 
+            
         return notebook
 
 
+    def save_notebooks(self, silent=False):
+        """Save all opened notebooks"""
+
+        # clear all window and viewer info in notebooks
+        for notebook in self._notebooks.itervalues():
+            notebook.pref.clear("windows", "ids")
+            notebook.pref.clear("viewers", "ids")
+
+        # save all the windows
+        for window in self._windows:
+            window.save_notebook(silent=silent)
+
+        # save all the notebooks
+        for notebook in self._notebooks.itervalues():
+            notebook.save()
+
+        # let windows know about completed save
+        for window in self._windows:
+            window.update_title()
+    
+
+    def _on_closing_notebook(self, notebook, save):
+        """
+        Callback for when notebook is about to close
+        """
+        keepnote.KeepNote._on_closing_notebook(self, notebook, save)
+        
+        if save:
+            self.save()
+
+        for window in self._windows:
+            window.close_notebook(notebook)
+        
+
+    def goto_nodeid(self, nodeid):
+        """
+        Open a node by nodeid
+        """
+        for window in self.get_windows():
+            notebook = window.get_notebook()
+            if not notebook:
+                continue
+            node = notebook.get_node_by_id(nodeid)
+            if node:
+                window.get_viewer().goto_node(node)
+                break
+
+
+
+    #=====================================
+    # auto-save
+
+    def begin_auto_save(self):
+        """Begin autosave callbacks"""
+
+        if self.pref.get("autosave"):
+            self._auto_saving = True
+
+            if not self._auto_save_registered:
+                self._auto_save_registered = True
+                gobject.timeout_add(self.pref.get("autosave_time"), 
+                                    self.auto_save)
+        else:
+            self._auto_saving = False
+
+        
+    def end_auto_save(self):
+        """Stop autosave"""
+        self._auto_saving = False
+
+
+    def auto_save(self):
+        """Callback for autosaving"""
+        
+        self._auto_saving = self.pref.get("autosave")
+
+        # NOTE: return True to activate next timeout callback
+        if not self._auto_saving:
+            self._auto_save_registered = False
+            return False
+
+        # don't do autosave if it is paused
+        if self._auto_save_pause > 0:
+            return True
+
+        self.save(True)
+
+        return True
+    
+
+    def pause_auto_save(self, pause):
+        """Pauses autosaving"""
+        self._auto_save_pause += 1 if pause else -1
+        
+
     #===========================================
     # node icons
 
@@ -497,7 +748,6 @@ class KeepNote (keepnote.KeepNote):
     #==================================
     # file attachment
 
-
     def on_attach_file(self, node=None, parent_window=None):
 
         dialog = FileChooserDialog(
@@ -508,6 +758,7 @@ class KeepNote (keepnote.KeepNote):
             app=self,
             persistent_path="attach_file_path")
         dialog.set_default_response(gtk.RESPONSE_OK)
+        dialog.set_select_multiple(True)
 
         # setup preview
         preview = gtk.Image()
@@ -517,23 +768,61 @@ class KeepNote (keepnote.KeepNote):
         response = dialog.run()
 
         if response == gtk.RESPONSE_OK:
-            if dialog.get_filename():
-                filename = unicode_gtk(dialog.get_filename())
-                try:
-                    notebooklib.attach_file(filename, node)
-                except Exception, e:
-                    self.error(_("Error while attaching file '%s'." % filename),
-                               e, sys.exc_info()[2])
+            if len(dialog.get_filenames()) > 0:
+                filenames = map(unicode_gtk, dialog.get_filenames())
+                self.attach_files(filenames, node,
+                                  parent_window=parent_window)
 
         dialog.destroy()
 
 
+    def attach_file(self, filename, parent, index=None, 
+                    parent_window=None):
+        self.attach_files([filename], parent, index, parent_window)
+
+
+    def attach_files(self, filenames, parent, index=None, 
+                     parent_window=None):
+
+        if parent_window is None:
+            parent_window = self.get_current_window()
+
+
+        #def func(task):
+        #    for filename in filenames:
+        #        task.set_message(("detail", _("attaching %s") % 
+        #                          os.path.basename(filename)))
+        #        notebooklib.attach_file(filename, parent, index)
+        #        if not task.is_running():
+        #            task.abort()
+        #task = tasklib.Task(func)
+
+        try:
+            for filename in filenames:
+                notebooklib.attach_file(filename, parent, index)
+            
+            #dialog = keepnote.gui.dialog_wait.WaitDialog(parent_window)
+            #dialog.show(_("Attach File"), _("Attaching files to notebook."), 
+            #            task, cancel=False)
+
+            #if task.aborted():
+            #    raise task.exc_info()[1]
+            
+        except Exception, e:
+            if len(filenames) > 1:
+                self.error(_("Error while attaching files %s." % 
+                             ", ".join(["'%s'" % f for f in filenames])),
+                           e, sys.exc_info()[2])
+            else:
+                self.error(_("Error while attaching file '%s'." % filenames[0]),
+                       e, sys.exc_info()[2])
+
+
     #==================================
     # misc GUI
 
     def focus_windows(self):
         """Focus all open windows on desktop"""
-
         for window in self._windows:
             window.restore_window()
 
@@ -596,7 +885,8 @@ class KeepNote (keepnote.KeepNote):
 
     def quit(self):
         """Quit the gtk event loop"""
-        
+        keepnote.KeepNote.quit(self)
+
         gtk.accel_map_save(get_accel_file())
         gtk.main_quit()
 
@@ -604,12 +894,11 @@ class KeepNote (keepnote.KeepNote):
     #===================================
     # callbacks
 
-
     def _on_window_close(self, window, event):
         """Callback for window close event"""
 
         if window in self._windows:
-            for ext in self.iter_extensions():
+            for ext in self.get_enabled_extensions():
                 try:
                     if isinstance(ext, keepnote.gui.extension.Extension):
                         ext.on_close_window(window)
@@ -635,12 +924,11 @@ class KeepNote (keepnote.KeepNote):
     #====================================
     # extension methods
 
-
     def init_extensions_windows(self, windows=None, exts=None):
         """Initialize all extensions for a window"""
         
         if exts is None:
-            exts = self.iter_extensions()
+            exts = self.get_enabled_extensions()
 
         if windows is None:
             windows = self.get_windows()
@@ -667,22 +955,21 @@ class KeepNote (keepnote.KeepNote):
 
             if len(new_exts) > 0:
                 self.message(_("Extension \"%s\" is now installed.") %
-                               filename, _("Install Sucessful"))
+                               filename, _("Install Successful"))
                 return True
 
         return False
 
 
-
-    def uninstall_extension(self, ext):
+    def uninstall_extension(self, ext_key):
         """Install a new extension"""
 
         if self.ask_yes_no(_("Do you want to uninstall the extension \"%s\"?") %
-                           ext.name, _("Extension Uninstall")):
-            if keepnote.KeepNote.uninstall_extension(self, ext):
+                           ext_key, _("Extension Uninstall")):
+            if keepnote.KeepNote.uninstall_extension(self, ext_key):
                 self.message(_("Extension \"%s\" is now uninstalled.") %
-                             ext.name, 
-                             _("Uninstall Sucessful"))
+                             ext_key, 
+                             _("Uninstall Successful"))
                 return True
 
         return False
