@@ -39,16 +39,22 @@ from gtk import gdk
 # keepnote imports
 import keepnote
 from keepnote import unicode_gtk
-from keepnote.notebook import NoteBookError, NoteBookTrash
+from keepnote.notebook import NoteBookError
+from keepnote.gui.icons import get_node_icon
 from keepnote.gui.treemodel import \
-     get_path_from_node
+     get_path_from_node, iter_children
 
 _ = keepnote.translate
 
 
 
-CLIPBOARD_NAME = "CLIPBOARD"     
-MIME_NODE = "application/x-keepnote-node"
+if keepnote.get_platform() == "darwin":
+    CLIPBOARD_NAME = gdk.SELECTION_PRIMARY
+else:
+    CLIPBOARD_NAME = "CLIPBOARD"
+MIME_NODE_COPY = "application/x-keepnote-node-copy"
+MIME_TREE_COPY = "application/x-keepnote-tree-copy"
+MIME_NODE_CUT = "application/x-keepnote-node-cut"
 
 # treeview drag and drop config
 DROP_URI = ("text/uri-list", 0, 1)
@@ -70,6 +76,7 @@ def parse_utf(text):
         len(text) > 3 and text[3] == '\x00'):
         return text.decode("utf16")
     else:
+        text = text.replace("\x00", "")
         return unicode(text, "utf8")
 
 
@@ -90,6 +97,8 @@ def compute_new_path(model, target, drop_position):
             str(drop_position))
 
 
+
+
 class KeepNoteBaseTreeView (gtk.TreeView):
     """Base class for treeviews of a NoteBook notes"""
 
@@ -107,9 +116,11 @@ class KeepNoteBaseTreeView (gtk.TreeView):
         self.__suppress_sel = False
         self._node_col = None
         self._get_icon = None
+        self._get_node = self._get_node_default
 
         self._menu = None
 
+        
         # selection
         self.get_selection().connect("changed", self.__on_select_changed)
         self.get_selection().connect("changed", self.on_select_changed)
@@ -132,6 +143,9 @@ class KeepNoteBaseTreeView (gtk.TreeView):
 
         # clipboard
         self.connect("copy-clipboard", self._on_copy_node)
+        self.connect("copy-tree-clipboard", self._on_copy_tree)
+        self.connect("cut-clipboard", self._on_cut_node)
+        self.connect("paste-clipboard", self._on_paste_node)
 
         # drop and drop events
         self.connect("drag-begin", self._on_drag_begin)
@@ -153,14 +167,17 @@ class KeepNoteBaseTreeView (gtk.TreeView):
                                     gtk.gdk.ACTION_MOVE|
                                     gtk.gdk.ACTION_COPY|
                                     gtk.gdk.ACTION_LINK)
-        self.drag_dest_set(gtk.DEST_DEFAULT_HIGHLIGHT | gtk.DEST_DEFAULT_MOTION,
-                           [DROP_TREE_MOVE, DROP_URI],
-                           gtk.gdk.ACTION_DEFAULT|
-                           gtk.gdk.ACTION_MOVE|
-                           gtk.gdk.ACTION_COPY|
-                           gtk.gdk.ACTION_LINK|
-                           gtk.gdk.ACTION_PRIVATE|
-                           gtk.gdk.ACTION_ASK)
+
+        
+        self.drag_dest_set(
+            gtk.DEST_DEFAULT_HIGHLIGHT | gtk.DEST_DEFAULT_MOTION,
+            [DROP_TREE_MOVE, DROP_URI],
+            gtk.gdk.ACTION_DEFAULT|
+            gtk.gdk.ACTION_MOVE|
+            gtk.gdk.ACTION_COPY|
+            gtk.gdk.ACTION_LINK|
+            gtk.gdk.ACTION_PRIVATE|
+            gtk.gdk.ACTION_ASK)
 
     
     def set_master_node(self, node):
@@ -186,6 +203,20 @@ class KeepNoteBaseTreeView (gtk.TreeView):
                 self.model.set_notebook(notebook)
             
 
+    def set_get_node(self, get_node_func=None):
+
+        if get_node_func is None:
+            self._get_node = self._get_node_default
+        else:
+            self._get_node = get_node_func
+
+
+    def _get_node_default(self, nodeid):
+        if self._notebook is None:
+            return None
+        return self._notebook.get_node_by_id(nodeid)
+        
+
     def set_model(self, model):
         """Set the model for the view"""
         
@@ -218,13 +249,14 @@ class KeepNoteBaseTreeView (gtk.TreeView):
 
             # init signals for model
             self.rich_model.set_notebook(self._notebook)
-            self.changed_start_id = self.rich_model.connect("node-changed-start",
-                                                   self._on_node_changed_start)
-            self.changed_end_id = self.rich_model.connect("node-changed-end",
-                                                 self._on_node_changed_end)
-            self._node_col = self.rich_model.get_node_column()
+            self.changed_start_id = self.rich_model.connect(
+                "node-changed-start", self._on_node_changed_start)
+            self.changed_end_id = self.rich_model.connect(
+                "node-changed-end", self._on_node_changed_end)
+            self._node_col = self.rich_model.get_node_column_pos()
             self._get_icon = lambda row: \
-                             self.model.get_value(row, self.rich_model.get_column_by_name("icon").pos)
+                self.model.get_value(
+                row, self.rich_model.get_column_by_name("icon").pos)
 
                 
             self.insert_id = self.model.connect("row-inserted",
@@ -232,8 +264,7 @@ class KeepNoteBaseTreeView (gtk.TreeView):
             self.delete_id = self.model.connect("row-deleted",
                                                 self.on_row_deleted)
             self.has_child_id = self.model.connect(
-                "row-has-child-toggled",
-                self.on_row_has_child_toggled)
+                "row-has-child-toggled", self.on_row_has_child_toggled)
 
 
     def set_popup_menu(self, menu):
@@ -254,7 +285,11 @@ class KeepNoteBaseTreeView (gtk.TreeView):
             return False
         
         path = path[0]
-        self.get_selection().select_path(path)
+
+        if not self.get_selection().path_is_selected(path):
+            self.get_selection().unselect_all()
+            self.get_selection().select_path(path)
+
         self._menu.popup(None, None, None, button, time)
         self._menu.show()
         return True
@@ -280,26 +315,32 @@ class KeepNoteBaseTreeView (gtk.TreeView):
 
 
     def _on_node_changed_end(self, model, nodes):
-
+        
         # maintain proper expansion
         for node in nodes:
 
             if node == self._master_node:
                 for child in node.get_children():
                     if self.is_node_expanded(child):
-                        path = get_path_from_node(self.model, child, self.rich_model.get_node_column())
+                        path = get_path_from_node(
+                            self.model, child, 
+                            self.rich_model.get_node_column_pos())
                         self.expand_row(path, False)
             else:
-                path = get_path_from_node(self.model, node,
-                                          self.rich_model.get_node_column())
+                try:
+                    path = get_path_from_node(
+                        self.model, node,
+                        self.rich_model.get_node_column_pos())
+                except:
+                    path = None
                 if path is not None:
                     parent = node.get_parent()
 
                     # NOTE: parent may lose expand state if it has one child
                     # therefore, we should expand parent if it exists and is
                     # visible (i.e. len(path)>1) in treeview
-                    if parent and self.is_node_expanded(parent) and \
-                       len(path) > 1:
+                    if (parent and self.is_node_expanded(parent) and 
+                        len(path) > 1):
                         self.expand_row(path[:-1], False)
 
                     if self.is_node_expanded(node):
@@ -308,40 +349,34 @@ class KeepNoteBaseTreeView (gtk.TreeView):
         
         
         # if nodes still exist, and expanded, try to reselect them
-        deselect = False
-        if len(self.__sel_nodes2) > 0:
-            # TODO: only reselects one node
-            node = self.__sel_nodes2[0]
-
+        sel_count = 0
+        selection = self.get_selection()
+        for node in self.__sel_nodes2:
+            sel_count += 1
             if node.is_valid():
-                path2 = get_path_from_node(self.model, node,
-                                           self.rich_model.get_node_column())
-                if path2 is not None and \
-                   (len(path2) <= 1 or self.row_expanded(path2[:-1])):
+                path2 = get_path_from_node(
+                    self.model, node, self.rich_model.get_node_column_pos())
+                if (path2 is not None and 
+                    (len(path2) <= 1 or self.row_expanded(path2[:-1]))):
                     # reselect and scroll to node 
-                    self.set_cursor(path2)
-            else:
-                # emit de-selection
-                deselect = True
-        
+                    selection.select_path(path2)
+
+
         # restore scroll
         gobject.idle_add(lambda : self.scroll_to_point(*self.__scroll))
 
         # resume emitting selection changes
         self.__suppress_sel = False
 
-        if deselect:
+        # emit de-selection
+        if sel_count == 0:
             self.select_nodes([])
 
 
     def __on_select_changed(self, treeselect):
         """Keep track of which nodes are selected"""
-        model, paths = treeselect.get_selected_rows()
-
-        self.__sel_nodes = [self.model.get_value(self.model.get_iter(path),
-                                                 self._node_col)
-                            for path in paths]
 
+        self.__sel_nodes = self.get_selected_nodes()        
         if self.__suppress_sel:
             self.get_selection().stop_emission("changed")
     
@@ -404,21 +439,34 @@ class KeepNoteBaseTreeView (gtk.TreeView):
     def expand_node(self, node):
         """Expand a node in TreeView"""
         path = get_path_from_node(self.model, node,
-                                  self.rich_model.get_node_column())
+                                  self.rich_model.get_node_column_pos())
         if path is not None:
             self.expand_to_path(path)
 
+    def collapse_all_beneath(self, path):
+        """Collapse all children beneath a path"""
+
+        it = self.model.get_iter(path)
+        def walk(it):
+            for child in iter_children(self.model, it):
+                walk(child)
+            path2 = self.model.get_path(it)
+            self.collapse_row(path2)
+        walk(it)
+
+
 
     #===========================================
     # selection
 
     def select_nodes(self, nodes):
+        """Select nodes in treeview"""
 
         # NOTE: for now only select one node
         if len(nodes) > 0:
             node = nodes[0]
             path = get_path_from_node(self.model, node,
-                                      self.rich_model.get_node_column())
+                                      self.rich_model.get_node_column_pos())
             if path is not None:
                 if len(path) > 1:
                     self.expand_to_path(path[:-1])
@@ -430,12 +478,9 @@ class KeepNoteBaseTreeView (gtk.TreeView):
 
 
     def on_select_changed(self, treeselect): 
-        model, paths = treeselect.get_selected_rows()
+        """Callback for when selection changes"""
 
-        #print paths
-
-        nodes = [self.model.get_value(self.model.get_iter(path), self._node_col)
-                 for path in paths]
+        nodes = self.get_selected_nodes()
         self.emit("select-nodes", nodes)
         return True
     
@@ -443,18 +488,27 @@ class KeepNoteBaseTreeView (gtk.TreeView):
     def get_selected_nodes(self):
         """Returns a list of currently selected nodes"""
 
-        # TODO: handle multiple selection
-        model, it = self.get_selection().get_selected()        
-        if it is None:
-            #print "edit", self.editing
+        iters = self.get_selected_iters()
+        if len(iters) == 0:
             if self.editing:
                 node = self._get_node_from_path(self.editing)
                 if node:
                     return [node]
             return []
-        return [self.model.get_value(it, self._node_col)]
+        else:
+            return [self.model.get_value(it, self._node_col)
+                    for it in iters]
 
         
+    def get_selected_iters(self):
+        """Return a list of currently selected TreeIter's"""
+
+        iters = []
+        self.get_selection().selected_foreach(lambda model, path, it:
+                                                  iters.append(it))
+        return iters
+
+
     # TODO: add a reselect if node is deleted
     # select next sibling or parent
 
@@ -508,7 +562,7 @@ class KeepNoteBaseTreeView (gtk.TreeView):
         #    self.set_cursor((0,))
         
         path = get_path_from_node(self.model, node,
-                                  self.rich_model.get_node_column())
+                                  self.rich_model.get_node_column_pos())
         if path is not None:
             self.set_cursor(path)
             gobject.idle_add(lambda: self.scroll_to_cell(path))
@@ -523,11 +577,11 @@ class KeepNoteBaseTreeView (gtk.TreeView):
     def _on_copy_node(self, widget):
         """Copy a node onto the clipboard"""
         
-        nodes = widget.get_selected_nodes()
+        nodes = self.get_selected_nodes()
         if len(nodes) > 0:
             clipboard = self.get_clipboard(selection=CLIPBOARD_NAME)
             
-            targets = [(MIME_NODE, gtk.TARGET_SAME_APP, -1),
+            targets = [(MIME_NODE_COPY, gtk.TARGET_SAME_APP, -1),
                        ("text/html", 0, -1),
                        ("text/plain", 0, -1)]
             
@@ -535,15 +589,85 @@ class KeepNoteBaseTreeView (gtk.TreeView):
                                     self._clear_selection_data,
                                     nodes)
 
+    def _on_copy_tree(self, widget):
+
+        nodes = self.get_selected_nodes()
+        if len(nodes) > 0:
+            clipboard = self.get_clipboard(selection=CLIPBOARD_NAME)
+            
+            targets = [(MIME_TREE_COPY, gtk.TARGET_SAME_APP, -1),
+                       (MIME_NODE_COPY, gtk.TARGET_SAME_APP, -1),
+                       ("text/html", 0, -1),
+                       ("text/plain", 0, -1)]
+            
+            clipboard.set_with_data(targets, self._get_selection_data, 
+                                    self._clear_selection_data,
+                                    nodes)        
+
+
+    def _on_cut_node(self, widget):
+        """Copy a node onto the clipboard"""
+        
+        nodes = widget.get_selected_nodes()
+        if len(nodes) > 0:
+            clipboard = self.get_clipboard(selection=CLIPBOARD_NAME)
+            
+            targets = [(MIME_NODE_CUT, gtk.TARGET_SAME_APP, -1),
+                       ("text/html", 0, -1),
+                       ("text/plain", 0, -1)]
+            
+            clipboard.set_with_data(targets, self._get_selection_data, 
+                                    self._clear_selection_data,
+                                    nodes)
+
+            self._fade_nodes(nodes)
+
+
+    def _on_paste_node(self, widget):
+        """Paste into the treeview"""
+        
+        clipboard = self.get_clipboard(selection=CLIPBOARD_NAME)
+        
+        targets = clipboard.wait_for_targets()
+        if targets is None:
+            # nothing on clipboard
+            return
+        targets = set(targets)
+        
+        if MIME_NODE_CUT in targets:
+            # request KEEPNOTE node objects
+            clipboard.request_contents(MIME_NODE_CUT, self._do_paste_nodes)
+
+        elif MIME_TREE_COPY in targets:
+            # request KEEPNOTE node objects
+            clipboard.request_contents(MIME_TREE_COPY, self._do_paste_nodes)
+
+        elif MIME_NODE_COPY in targets:
+            # request KEEPNOTE node objects
+            clipboard.request_contents(MIME_NODE_COPY, self._do_paste_nodes)
+
 
     def _get_selection_data(self, clipboard, selection_data, info, nodes):
         """Callback for when Clipboard needs selection data"""        
-
-        if MIME_NODE in selection_data.target:
+        
+        if MIME_NODE_CUT in selection_data.target:
             # set nodes
-            selection_data.set(MIME_NODE, 8, 
+            selection_data.set(MIME_NODE_CUT, 8, 
                                ";".join([node.get_attr("nodeid") 
                                          for node in nodes]))
+
+        elif MIME_TREE_COPY in selection_data.target:
+            # set nodes
+            selection_data.set(MIME_TREE_COPY, 8, 
+                               ";".join([node.get_attr("nodeid") 
+                                         for node in nodes]))
+
+        elif MIME_NODE_COPY in selection_data.target:
+            # set nodes
+            selection_data.set(MIME_NODE_COPY, 8, 
+                               ";".join([node.get_attr("nodeid") 
+                                         for node in nodes]))
+
             
         elif "text/html" in selection_data.target:
             # set html            
@@ -559,10 +683,76 @@ class KeepNoteBaseTreeView (gtk.TreeView):
                                               for node in nodes]))
 
     
+    def _do_paste_nodes(self, clipboard, selection_data, data):
+        """Paste nodes into treeview"""
+
+        if self._notebook is None:
+            return        
+
+        # find paste location
+        selected = self.get_selected_nodes()
+        if len(selected) > 0:
+            parent = selected[0]
+        else:
+            parent = self._notebook
+
+        
+        # find nodes to paste
+        nodeids = selection_data.data.split(";")
+        nodes = [self._get_node(nodeid) for nodeid in nodeids]
+
+        #nodes = [self._notebook.get_node_by_id(nodeid)
+        #         for nodeid in nodeids]
+
+        
+        if selection_data.target == MIME_NODE_CUT:
+            for node in nodes:
+                try:
+                    if node is not None:
+                        node.move(parent)
+                except:
+                    keepnote.log_error()
+
+        elif selection_data.target == MIME_TREE_COPY:
+            for node in nodes:
+                try:
+                    if node is not None:
+                        node.duplicate(parent, recurse=True)
+                except Exception, e:
+                    keepnote.log_error()
+
+        elif selection_data.target == MIME_NODE_COPY:
+            for node in nodes:
+                try:
+                    if node is not None:
+                        node.duplicate(parent)
+                except Exception, e:
+                    keepnote.log_error()
+            
+
+
     def _clear_selection_data(self, clipboard, data):
         """Callback for when Clipboard contents are reset"""
-        pass
+
+        self._clear_fading()
+        
+
+    #============================================
+    # node fading
     
+    def _clear_fading(self):
+        """Clear faded nodes"""
+        
+        nodes = list(self.rich_model.fades)
+        self.rich_model.fades.clear()
+        self._notebook.notify_changes(nodes, False)
+
+    def _fade_nodes(self, nodes):
+
+        self.rich_model.fades.clear()        
+        for node in nodes:
+            self.rich_model.fades.add(node)
+            node.notify_change(False)
 
     
     #=============================================
@@ -577,10 +767,15 @@ class KeepNoteBaseTreeView (gtk.TreeView):
 
     
     def get_drag_node(self):
-        # TODO: handle multiple selection
-        model, source = self.get_selection().get_selected()
-        #source_path = model.get_path(source)
-        return self.model.get_value(source, self._node_col)
+        iters = self.get_selected_iters()
+        if len(iters) == 0:
+            return None
+        return self.model.get_value(iters[0], self._node_col)
+
+
+    def get_drag_nodes(self):
+        return [self.model.get_value(it, self._node_col)
+                for it in self.get_selected_iters()]
 
 
     #  drag and drop callbacks
@@ -642,9 +837,12 @@ class KeepNoteBaseTreeView (gtk.TreeView):
         """Callback for beginning of drag and drop"""
         self.stop_emission("drag-begin")
 
-        # TODO: handle multiple selection
-        # get the selection
-        model, source = self.get_selection().get_selected()
+        iters = self.get_selected_iters()
+        if len(iters) == 0:
+            return
+
+        # use first selected item for icon
+        source = iters[0]
 
         # setup the drag icon
         if self._get_icon:
@@ -655,20 +853,24 @@ class KeepNoteBaseTreeView (gtk.TreeView):
         # clear the destination row
         self._dest_row = None
 
+        self.cancel_editing()
+
         self._is_dragging = True
         self._drag_count = 0
         gobject.timeout_add(200, self._on_drag_timer)
 
         
-    def _on_drag_motion(self, treeview, drag_context, x, y, eventtime):
+    def _on_drag_motion(self, treeview, drag_context, x, y, eventtime,
+                        stop_emit=True):
         """
         Callback for drag motion.
         Indicate which drops are allowed (cannot drop into descendant).
         Also record the destination for later use.
-        """        
-
+        """
+        
         # override gtk's default drag motion code
-        self.stop_emission("drag-motion")
+        if stop_emit:
+            self.stop_emission("drag-motion")
 
         # if reordering is disabled then terminate the drag
         if self._reorder == REORDER_NONE:
@@ -688,10 +890,16 @@ class KeepNoteBaseTreeView (gtk.TreeView):
             if "drop_node" in drag_context.targets:
                 # get source
                 source_widget = drag_context.get_source_widget()
-                source_node = source_widget.get_drag_node()
+                source_nodes = source_widget.get_drag_nodes()
             
                 # determine if drag is allowed
-                if self._drop_allowed(source_node, target_node, drop_position):
+                allow = True
+                for source_node in source_nodes:
+                    if not self._drop_allowed(source_node, target_node, 
+                                              drop_position):
+                        allow = False
+                
+                if allow:
                     self.set_drag_dest_row(target_path, drop_position)
                     self._dest_row = target_path, drop_position
                     drag_context.drag_status(gdk.ACTION_MOVE, eventtime)
@@ -753,11 +961,13 @@ class KeepNoteBaseTreeView (gtk.TreeView):
         # override gtk's data get code
         self.stop_emission("drag-data-get")
 
-        # TODO: handle multiple selection
-        # set the source path into the selection
-        model, source = self.get_selection().get_selected()
-        source_path = model.get_path(source)
-        selection_data.tree_set_row_drag_data(model, source_path)
+        # TODO: think more about what data to actually set for 
+        # tree_set_row_drag_data()
+        iters = self.get_selected_iters()
+        if len(iters) > 0:
+            source = iters[0]
+            source_path = self.model.get_path(source)
+            selection_data.tree_set_row_drag_data(self.model, source_path)
     
     
     def _on_drag_data_received(self, treeview, drag_context, x, y,
@@ -766,10 +976,15 @@ class KeepNoteBaseTreeView (gtk.TreeView):
         """
         Callback for when data is received from source widget
         """
-
+        
         # override gtk's data received code
         self.stop_emission("drag-data-received")
 
+        # NOTE: force one more call to motion in order, since Windows ignores
+        # cross app drag calls
+        self._on_drag_motion(treeview, drag_context, x, y, eventtime,
+                             stop_emit=False)
+
         
         # if no destination, give up.  Occurs when drop is not allowed
         if self._dest_row is None:
@@ -793,12 +1008,15 @@ class KeepNoteBaseTreeView (gtk.TreeView):
 
                 uris = parse_utf(selection_data.data)
                 uris = [x for x in (urllib.unquote(uri.strip())
-                                for uri in uris.split("\n"))
+                                    for uri in uris.split("\n"))
                         if len(x) > 0 and x[0] != "#"]
 
                 for uri in reversed(uris):
                     if uri.startswith("file://"):
                         uri = uri[7:]
+                        if keepnote.get_platform() == "windows":
+                            # remove one more '/' for windows
+                            uri  = uri[1:]
                     self.emit("drop-file", parent, new_path[-1], uri)
             drag_context.finish(True, False, eventtime)
             
@@ -832,32 +1050,36 @@ class KeepNoteBaseTreeView (gtk.TreeView):
 
         # get source
         source_widget = drag_context.get_source_widget()
-        source_node = source_widget.get_drag_node()
-
-        # determine if drop is allowed
-        if not self._drop_allowed(source_node, target_node, drop_position):
+        source_nodes = source_widget.get_drag_nodes()
+        if len(source_nodes) == 0:
             drag_context.finish(False, False, eventtime)
             return
-
-        # determine new parent
-        new_parent_path = new_path[:-1]
-        new_parent = self._get_node_from_path(new_parent_path)
-
         
-        # perform move in notebook model
-        try:
-            source_node.move(new_parent, new_path[-1])
-        except NoteBookError, e:
-            drag_context.finish(False, False, eventtime)
-            self.emit("error", e.msg, e)
-            return
+        # determine if drop is allowed
+        for source_node in source_nodes:
+            if not self._drop_allowed(source_node, target_node, drop_position):
+                drag_context.finish(False, False, eventtime)
+                continue
+
+            # determine new parent
+            new_parent_path = new_path[:-1]
+            new_parent = self._get_node_from_path(new_parent_path)
+
+            # perform move in notebook model
+            try:
+                source_node.move(new_parent, new_path[-1])
+            except NoteBookError, e:
+                # TODO: think about whether finish should always be false
+                drag_context.finish(False, False, eventtime)
+                self.emit("error", e.msg, e)
+                return
 
         # re-establish selection on source node
-        self.emit("goto-node", source_node)
+        self.emit("goto-node", source_nodes[0])
 
         # notify that drag was successful
         drag_context.finish(True, True, eventtime)
-    
+  
         
     def _drop_allowed(self, source_node, target_node, drop_position):
         """Determine if drop is allowed"""
@@ -899,6 +1121,9 @@ gobject.signal_new("goto-parent-node", KeepNoteBaseTreeView,
 gobject.signal_new("copy-clipboard", KeepNoteBaseTreeView,
                    gobject.SIGNAL_RUN_LAST, 
                    gobject.TYPE_NONE, ())
+gobject.signal_new("copy-tree-clipboard", KeepNoteBaseTreeView,
+                   gobject.SIGNAL_RUN_LAST, 
+                   gobject.TYPE_NONE, ())
 gobject.signal_new("cut-clipboard", KeepNoteBaseTreeView,
                    gobject.SIGNAL_RUN_LAST, 
                    gobject.TYPE_NONE, ())
