@@ -127,6 +127,42 @@ def iter_buffer_contents(textbuffer, start=None, end=None,
             yield ("end", end, tag)
 
 
+def iter_buffer_anchors(textbuffer, start=None, end=None):
+    """Iterate over the anchors of a textbuffer
+
+    textbuffer -- buffer to iterate over
+    start      -- starting position (TextIter)
+    end        -- ending position (TextIter)
+    """
+
+    # initialize iterators
+    if start is None:
+        it = textbuffer.get_start_iter()
+    else:
+        it = start.copy()
+
+    if end is None:
+        end = textbuffer.get_end_iter()
+        
+    while True:
+        # find next achor
+        ret = it.forward_search(ANCHOR_CHAR, (), end)
+        if not ret:                
+            break
+
+        # anchor found
+        a, b = ret
+        anchor = a.get_child_anchor()
+            
+        # yield anchor
+        if anchor is not None:
+            yield ("anchor", a, (anchor, anchor.get_widgets()))
+
+        # advance it past anchor
+        it = b
+
+
+
 def buffer_contents_iter_to_offset(contents):
     """Converts to iters of a content list to offsets"""
     
@@ -360,6 +396,12 @@ def buffer_contents_apply_tags(textbuffer, contents):
             textbuffer.apply_tag(param, start, end)
 
 
+def sanitize_text(text):
+    if "\r\n" in text:
+        # windows line feed
+        return text.replace("\r\n", "\n")
+    else:
+        return text
 
 
 #=============================================================================
@@ -409,6 +451,9 @@ class paragraph_iter (object):
             self.buf.delete_mark(self.end_mark)
 
     def __iter__(self):
+        self.pos = self.buf.get_iter_at_mark(self.pos_mark)
+        self.end = self.buf.get_iter_at_mark(self.end_mark)
+
         while self.pos.compare(self.end) == -1:
             self.buf.move_mark(self.pos_mark, self.pos)
             yield self.pos
@@ -486,13 +531,28 @@ class TextDom (Dom):
     """A text object in a DOM"""
     def __init__(self, text):
         Dom.__init__(self)
-        self.text = text
+        self.lst = [text]
+
+    def append(self, text):
+        self.lst.append(text)
+
+    def get(self):
+        if len(self.lst) == 1:
+            return self.lst[0]
+        else:
+            t = "".join(self.lst)
+            self.lst = [t]
+            return t
+        
+
+    def set(self, text):
+        self.lst = [text]
 
     def display(self, indent=0):
-        self.display_indent(indent, "TextDom '%s'" % self.text)
+        self.display_indent(indent, "TextDom '%s'" % self.get())
 
     def visit_contents(self, visit):
-        visit("text", None, self.text)
+        visit("text", None, self.get())
     
 
 class AnchorDom (Dom):
