@@ -1,7 +1,7 @@
 """
 
     KeepNote
-    Graphical User Interface for KeepNote Application
+    Classic three-paned viewer for KeepNote.
 
 """
 
@@ -43,7 +43,7 @@ import gobject
 # keepnote imports
 import keepnote
 from keepnote import unicode_gtk, KeepNoteError
-from keepnote.notebook import NoteBookTrash
+from keepnote.notebook import NoteBookError
 from keepnote.gui import \
     add_actions, \
     dialog_image_resize, \
@@ -61,9 +61,11 @@ from keepnote.gui import richtext
 from keepnote.gui.richtext import RichTextView, RichTextImage, RichTextError
 from keepnote.gui.treeview import KeepNoteTreeView
 from keepnote.gui.listview import KeepNoteListView
-from keepnote.gui.editor import KeepNoteEditor, EditorMenus
+from keepnote.gui.editor import KeepNoteEditor
+from keepnote.gui.editor_richtext import RichTextEditor
+from keepnote.gui.editor_text import TextEditor
+from keepnote.gui.editor_multi import ContentEditor
 from keepnote.gui.icon_menu import IconMenu
-from keepnote.gui.link_editor import LinkEditor
 from keepnote import notebook as notebooklib
 from keepnote.gui.treemodel import iter_children
 from keepnote.gui.viewer import Viewer
@@ -73,13 +75,19 @@ from keepnote.gui.icons import \
 _ = keepnote.translate
 
 
+DEFAULT_VSASH_POS = 200
+DEFAULT_HSASH_POS = 200
+DEFAULT_VIEW_MODE = "vertical"
+
+
 
 class ThreePaneViewer (Viewer):
     """A viewer with a treeview, listview, and editor"""
 
-    def __init__(self, app, main_window):
-        Viewer.__init__(self, app, main_window)
-
+    def __init__(self, app, main_window, viewerid=None):
+        Viewer.__init__(self, app, main_window, viewerid,
+                        viewer_name="three_pane_viewer")
+        self._ui_ready = False
 
         # node selections        
         self._current_page = None     # current page in editor
@@ -87,23 +95,28 @@ class ThreePaneViewer (Viewer):
         self._queue_list_select = []  # nodes to select in listview after treeview change
         self._new_page_occurred = False
         self.back_button = None
+        self._view_mode = DEFAULT_VIEW_MODE
 
         self.connect("history-changed", self._on_history_changed)
 
         #=========================================
         # widgets
-        
+
+
         # treeview
         self.treeview = KeepNoteTreeView()
+        self.treeview.set_get_node(self._app.get_node)
         self.treeview.connect("select-nodes", self._on_tree_select)
         self.treeview.connect("delete-node", self.on_delete_node)
         self.treeview.connect("error", lambda w,t,e: self.emit("error", t, e))
         self.treeview.connect("edit-title", self._on_edit_title)
         self.treeview.connect("goto-node", self.on_goto_node)
+        self.treeview.connect("activate-node", self.on_activate_node)
         self.treeview.connect("drop-file", self._on_attach_file)
         
         # listview
         self.listview = KeepNoteListView()
+        self.listview.set_get_node(self._app.get_node)
         self.listview.connect("select-nodes", self._on_list_select)
         self.listview.connect("delete-node", self.on_delete_node)
         self.listview.connect("goto-node", self.on_goto_node)
@@ -116,12 +129,17 @@ class ThreePaneViewer (Viewer):
         self.listview.on_status = self.set_status  # TODO: clean up
         
         # editor
-        self.editor = KeepNoteEditor(self._app)
-        self.editor_menus = EditorMenus(self.editor)
-        self.editor.connect("make-link", self._on_make_link)
+        #self.editor = KeepNoteEditor(self._app)
+        #self.editor = RichTextEditor(self._app)
+        self.editor = ContentEditor(self._app)
+        rich_editor = RichTextEditor(self._app)
+        self.editor.add_editor("text/xhtml+xml", rich_editor)
+        self.editor.add_editor("text", TextEditor(self._app))
+        self.editor.set_default_editor(rich_editor)
+
+        self.editor.connect("view-node", self._on_editor_view_node)
         self.editor.connect("child-activated", self._on_child_activated)
         self.editor.connect("visit-node", lambda w, n: self.goto_node(n, False))
-        self.editor.connect("font-change", self.editor_menus.on_font_change)
         self.editor.connect("error", lambda w,t,e: self.emit("error", t, e))
         self.editor.connect("window-request", lambda w,t: 
                             self.emit("window-request", t))
@@ -130,14 +148,6 @@ class ThreePaneViewer (Viewer):
         self.editor_pane = gtk.VBox(False, 5)
         self.editor_pane.pack_start(self.editor, True, True, 0)
 
-        self.link_editor = LinkEditor()
-        self.link_editor.set_textview(self.editor.get_textview())
-        self.editor.connect("font-change", self.link_editor.on_font_change)
-        self.editor.connect("view-node", self._on_editor_view_node)
-        self.editor_pane.pack_start(self.link_editor, False, True, 0)
-        self.link_editor.set_search_nodes(self.search_nodes)
-
-        self._main_window.make_image_menu(self.editor.get_textview().get_image_menu())
 
         #=====================================
         # layout
@@ -147,13 +157,13 @@ class ThreePaneViewer (Viewer):
         # create a horizontal paned widget
         self.hpaned = gtk.HPaned()
         self.pack_start(self.hpaned, True, True, 0)
-        self.hpaned.set_position(keepnote.DEFAULT_HSASH_POS)
+        self.hpaned.set_position(DEFAULT_HSASH_POS)
 
                 
         # layout major widgets
         self.paned2 = gtk.VPaned()
         self.hpaned.add2(self.paned2)
-        self.paned2.set_position(keepnote.DEFAULT_VSASH_POS)
+        self.paned2.set_position(DEFAULT_VSASH_POS)
         
         # treeview and scrollbars
         sw = gtk.ScrolledWindow()
@@ -168,102 +178,117 @@ class ThreePaneViewer (Viewer):
         self.listview_sw.set_shadow_type(gtk.SHADOW_IN)
         self.listview_sw.add(self.listview)
         self.paned2.add1(self.listview_sw)
+        #self.paned2.child_set_property(self.listview_sw, "shrink", True)
         
         # layout editor
         self.paned2.add2(self.editor_pane)
-
-        #self.show_all()
+        
         self.treeview.grab_focus()
 
 
     def set_notebook(self, notebook):
         """Set the notebook for the viewer"""
-        
+
+        # add/remove reference to notebook
+        self._app.ref_notebook(notebook)
+        if self._notebook is not None:
+            self._app.unref_notebook(self._notebook)
+
+        # deregister last notebook, if it exists
+        if self._notebook:
+            self._notebook.node_changed.remove(
+                self.on_notebook_node_changed)
+
+        # setup listeners
+        if notebook:
+            notebook.node_changed.add(self.on_notebook_node_changed)
+
+
+        # set notebook
         self._notebook = notebook
         self.editor.set_notebook(notebook)
         self.listview.set_notebook(notebook)
         self.treeview.set_notebook(notebook)
-
+        
         if self.treeview.get_popup_menu():
             self.treeview.get_popup_menu().iconmenu.set_notebook(notebook)
             self.listview.get_popup_menu().iconmenu.set_notebook(notebook)
 
         # restore selections
-        if self._notebook:
-            nodes = [node for node in (
-                    self._notebook.get_node_by_id(i)
-                    for i in self._notebook.pref.selected_treeview_nodes)
-                     if node is not None]
-            self.treeview.select_nodes(nodes)
-            nodes = [node for node in (
-                    self._notebook.get_node_by_id(i)
-                    for i in self._notebook.pref.selected_listview_nodes)
-                     if node is not None]
-            self.listview.select_nodes(nodes)
-
+        self._load_selections()
 
+        # put focus on treeview
         self.treeview.grab_focus()
 
 
     def load_preferences(self, app_pref, first_open=False):
         """Load application preferences"""
 
-        self.set_view_mode(app_pref.view_mode)
-        self.paned2.set_position(app_pref.vsash_pos)
-        self.hpaned.set_position(app_pref.hsash_pos)
+        p = app_pref.get("viewers", "three_pane_viewer", define=True)
+        self.set_view_mode(p.get("view_mode", DEFAULT_VIEW_MODE))
+        self.paned2.set_property("position-set", True)
+        self.hpaned.set_property("position-set", True)
+        self.paned2.set_position(p.get("vsash_pos", DEFAULT_VSASH_POS))
+        self.hpaned.set_position(p.get("hsash_pos", DEFAULT_HSASH_POS))
 
-        self.listview.set_date_formats(app_pref.timestamp_formats)
-        self.listview.set_rules_hint(app_pref.listview_rules)        
+
+        self.listview.set_date_formats(app_pref.get("timestamp_formats"))
+        self.listview.set_rules_hint(
+            app_pref.get("look_and_feel", "listview_rules",
+                         default=True))
         try:
             # if this version of GTK doesn't have tree-lines, ignore it
-            self.treeview.set_property("enable-tree-lines",
-                                       app_pref.treeview_lines)
+            self.treeview.set_property(
+                "enable-tree-lines",
+                app_pref.get("look_and_feel", "treeview_lines", default=True))
         except:
             pass
-
-        self.editor_menus.enable_spell_check(self._app.pref.spell_check)
-
+        
+        self.editor.load_preferences(app_pref, first_open)
+        
         # reload ui
-        self.remove_ui(self._main_window)
-        self.add_ui(self._main_window)
-
-        
-        
+        if self._ui_ready:
+            self.remove_ui(self._main_window)
+            self.add_ui(self._main_window)
 
 
     def save_preferences(self, app_pref):
         """Save application preferences"""
         
-        app_pref.vsash_pos = self.paned2.get_position()
-        app_pref.hsash_pos = self.hpaned.get_position()
-        
-        # record state in preferences
-        app_pref.spell_check = self.editor.get_textview().is_spell_check_enabled()
+        p = app_pref.get("viewers", "three_pane_viewer")
+        p["view_mode"] = self._view_mode
+        p["vsash_pos"] = self.paned2.get_position()
+        p["hsash_pos"] = self.hpaned.get_position()
 
+        self.editor.save_preferences(app_pref)
 
 
     def save(self):
         """Save the current notebook"""
 
         self.editor.save()
+        self._save_selections()
+               
+
+    def on_notebook_node_changed(self, nodes):
+        """Callback for when notebook node is changed"""
+
+        #if self._current_page in nodes:
+        #    self.emit("current-node", self._current_page)
+        self.emit("modified", True)
 
-        # save selections
-        self._notebook.pref.selected_treeview_nodes = [
-            node.get_attr("nodeid")
-            for node in self.treeview.get_selected_nodes()]
-        self._notebook.pref.selected_listview_nodes = [
-            node.get_attr("nodeid")
-            for node in self.listview.get_selected_nodes()]
-        self._notebook.set_preferences_dirty()
-        
 
     def undo(self):
         """Undo the last action in the viewer"""
-        self.editor.get_textview().undo()
+        self.editor.undo()        
 
     def redo(self):
         """Redo the last action in the viewer"""
-        self.editor.get_textview().redo()
+        self.editor.redo()
+
+
+    def get_editor(self):
+        return self.editor
 
 
     def set_status(self, text, bar="status"):
@@ -279,6 +304,8 @@ class ThreePaneViewer (Viewer):
             "horizontal"
         """
 
+        vsash = self.paned2.get_position()
+
         # detach widgets
         self.paned2.remove(self.listview_sw)
         self.paned2.remove(self.editor_pane)
@@ -291,8 +318,8 @@ class ThreePaneViewer (Viewer):
         else:
             # create a horizontal paned widget
             self.paned2 = gtk.HPaned()
-                    
-        self.paned2.set_position(self._app.pref.vsash_pos)
+        
+        self.paned2.set_position(vsash)
         self.paned2.show()        
         
         self.hpaned.add2(self.paned2)
@@ -302,9 +329,46 @@ class ThreePaneViewer (Viewer):
         self.paned2.add2(self.editor_pane)
 
         # record preference
-        if mode != self._app.pref.view_mode:
-            self._app.pref.view_mode = mode
-            self._app.pref.changed.notify()
+        self._view_mode = mode
+
+
+
+    def _load_selections(self):
+        """Load previous node selections from notebook preferences"""
+        
+        if self._notebook:
+            info = self._notebook.pref.get("viewers", "ids", 
+                                           self._viewerid, define=True)
+
+            # load selections
+            nodes = [node for node in (
+                    self._notebook.get_node_by_id(i)
+                    for i in info.get(
+                        "selected_treeview_nodes", []))
+                     if node is not None]
+            self.treeview.select_nodes(nodes)
+            nodes = [node for node in (
+                    self._notebook.get_node_by_id(i)
+                    for i in info.get(
+                        "selected_listview_nodes", []))
+                     if node is not None]
+            self.listview.select_nodes(nodes)
+
+    def _save_selections(self):
+        """Save node selections into notebook preferences"""
+        
+        if self._notebook is not None:
+            info = self._notebook.pref.get("viewers", "ids", 
+                                           self._viewerid, define=True)
+            
+            # save selections
+            info["selected_treeview_nodes"] = [
+                node.get_attr("nodeid")
+                for node in self.treeview.get_selected_nodes()]
+            info["selected_listview_nodes"] = [
+                node.get_attr("nodeid")
+                for node in self.listview.get_selected_nodes()]
+            self._notebook.set_preferences_dirty()
 
 
     #===============================================
@@ -316,41 +380,25 @@ class ThreePaneViewer (Viewer):
         return self._current_page
 
 
-    def get_selected_nodes(self, widget="focus"):
+    def get_selected_nodes(self):
         """
-        Returns (nodes, widget) where 'nodes' are a list of selected nodes
-        in widget 'widget'
-
-        Wiget can be
-           listview -- nodes selected in listview
-           treeview -- nodes selected in treeview
-           focus    -- nodes selected in widget with focus
+        Returns  a list of selected nodes.
         """
-        
-        if widget == "focus":
-            if self.listview.is_focus():
-                widget = "listview"
-            elif self.treeview.is_focus():
-                widget = "treeview"
-            elif self.editor.is_focus():
-                widget = "listview"
-            else:
-                return ([], "")
 
-        if widget == "treeview":
-            nodes = self.treeview.get_selected_nodes()
-        elif widget == "listview":
-            nodes = self.listview.get_selected_nodes()
+        if self.treeview.is_focus():
+            return self.treeview.get_selected_nodes()
         else:
-            raise Exception("unknown widget '%s'" % widget)
-
-        return (nodes, widget)
+            nodes = self.listview.get_selected_nodes()
+            if len(nodes) == 0:
+                return self.treeview.get_selected_nodes()
+            else:
+                return nodes
 
 
     def _on_history_changed(self, viewer, history):
         """Callback for when node browse history changes"""
         
-        if self.back_button:
+        if self._ui_ready and self.back_button:
             self.back_button.set_sensitive(history.has_back())
             self.forward_button.set_sensitive(history.has_forward())
 
@@ -366,42 +414,46 @@ class ThreePaneViewer (Viewer):
             return default
     
 
-    def on_delete_node(self, widget="focus", nodes=None):
+    def on_delete_node(self, widget, nodes=None):
         """Callback for deleting a node"""
         
         # get node to delete
         if nodes is None:
-            nodes, widget = self.get_selected_nodes()
+            nodes = self.get_selected_nodes()
             
         if len(nodes) == 0:
             return
-        node = nodes[0]
+
         
         if self._main_window.confirm_delete_nodes(nodes):
-            # change selection            
-            widget = self.get_focused_widget()
-            parent = node.get_parent()
-            children = parent.get_children()
-            i = children.index(node)
-            if i < len(children) - 1:
-                widget.select_nodes([children[i+1]])
+            # change selection
+            if len(nodes) == 1:
+                node = nodes[0]
+                widget = self.get_focused_widget(self.listview)
+                parent = node.get_parent()
+                children = parent.get_children()
+                i = children.index(node)
+
+                if i < len(children) - 1:
+                    widget.select_nodes([children[i+1]])
+                else:
+                    widget.select_nodes([parent])
             else:
-                widget.select_nodes([parent])
+                widget = self.get_focused_widget(self.listview)
+                widget.select_nodes([])
 
             # perform delete
             try:
-                node.trash()
+                for node in nodes:
+                    node.trash()
             except NoteBookError, e:
                 self.emit("error", e.msg, e)
-
-
-    def _on_make_link(self, editor):
-        """Callback from editor to make a link"""
-        self.link_editor.edit()
     
     
     def _on_editor_view_node(self, editor, node):
         """Callback for when editor views a node"""
+
+        # record node in history
         self._history.add(node.get_attr("nodeid"))
         self.emit("history-changed", self._history)
 
@@ -409,8 +461,9 @@ class ThreePaneViewer (Viewer):
     def _on_child_activated(self, editor, textview, child):
         """Callback for when child widget in editor is activated"""
         
-        if isinstance(child, richtext.RichTextImage):
-            self._main_window.view_image(child.get_filename())
+        if self._current_page and isinstance(child, richtext.RichTextImage):
+            filename = self._current_page.get_file(child.get_filename())
+            self._app.run_external_app("image_viewer", filename)
 
         
     def _on_tree_select(self, treeview, nodes):
@@ -444,13 +497,15 @@ class ThreePaneViewer (Viewer):
             self._current_page = pages[0]
         else:
             self._current_page = None
-        
+
         try:
             self.editor.view_pages(pages)
         except RichTextError, e:
             self.emit("error", 
-                      "Could not load page '%s'." % pages[0].get_title(),
-                      e, sys.exc_info()[2])
+                      "Could not load page '%s'." % pages[0].get_title(), e)
+
+        self.emit("current-node", self._current_page)
+
 
     def on_goto_node(self, widget, node):
         """Focus view on a node"""
@@ -478,7 +533,7 @@ class ThreePaneViewer (Viewer):
         """Focus view on a node's parent"""
 
         if node is None:
-            nodes, widget = self.get_selected_nodes()
+            nodes = self.get_selected_nodes()
             if len(nodes) == 0:
                 return
             node = nodes[0]
@@ -502,16 +557,20 @@ class ThreePaneViewer (Viewer):
 
     def _on_attach_file(self, widget, parent, index, uri):
         """Attach document"""
+        self._app.attach_file(uri, parent, index)
 
-        try:
-            notebooklib.attach_file(uri, parent, index)
-        except Exception, e:
-            self.error(_("Error while attaching file '%s'." % uri),
-                       e, sys.exc_info()[2])
+
+    def _on_attach_file_menu(self):
+        """Callback for attach file action"""
+        
+        nodes = self.get_selected_nodes()
+        if len(nodes) > 0:
+            node = nodes[0]
+            self._app.on_attach_file(node, self.get_toplevel())
 
 
 
-    def new_node(self, kind, widget, pos):
+    def new_node(self, kind, pos, parent=None):
         """Add a new node to the notebook"""
         
         # TODO: think about where this goes
@@ -521,79 +580,73 @@ class ThreePaneViewer (Viewer):
 
         self.treeview.cancel_editing()
         self.listview.cancel_editing()
-
-        nodes, widget = self.get_selected_nodes(widget)
         
-        if len(nodes) == 1:
-            parent = nodes[0]
-        else:
-            parent = self._notebook
+        if parent is None:
+            nodes = self.get_selected_nodes()
+            if len(nodes) == 1:
+                parent = nodes[0]
+            else:
+                parent = self._notebook
         
-        if pos == "sibling" and parent.get_parent() is not None:
-            index = parent.get_attr("order") + 1
-            parent = parent.get_parent()
-        else:
-            index = None
+        node = Viewer.new_node(self, kind, pos, parent)
 
-
-        if kind == notebooklib.CONTENT_TYPE_DIR:
-            node = parent.new_child(notebooklib.CONTENT_TYPE_DIR,
-                                    notebooklib.DEFAULT_DIR_NAME,
-                                    index)
-        else:
-            node = parent.new_child(notebooklib.CONTENT_TYPE_PAGE,
-                                    notebooklib.DEFAULT_PAGE_NAME,
-                                    index)
-        
-        self._view_new_node(node, widget)
+        self._view_new_node(node)
 
         
-    def on_new_dir(self, widget="focus"):
+    def on_new_dir(self):
         """Add new folder near selected nodes"""
-        self.new_node(notebooklib.CONTENT_TYPE_DIR, widget, "sibling")
+        self.new_node(notebooklib.CONTENT_TYPE_DIR, "sibling")
         
     
-    def on_new_page(self, widget="focus"):
+    def on_new_page(self):
         """Add new page near selected nodes"""
-        self.new_node(notebooklib.CONTENT_TYPE_PAGE, widget, "sibling")
+        self.new_node(notebooklib.CONTENT_TYPE_PAGE, "sibling")
     
 
-    def on_new_child_page(self, widget="focus"):
+    def on_new_child_page(self):
         """Add new page as child of selected nodes"""
-        self.new_node(notebooklib.CONTENT_TYPE_PAGE, widget, "child")
+        self.new_node(notebooklib.CONTENT_TYPE_PAGE, "child")
 
 
-    def _view_new_node(self, node, widget):
+    def _view_new_node(self, node):
         """View a node particular widget"""
         
         self._new_page_occurred = True
         
-        if widget == "treeview":
-            self.treeview.expand_node(node.get_parent())
+        self.goto_node(node)
+
+        if node in self.treeview.get_selected_nodes():
             self.treeview.edit_node(node)
-        elif widget == "listview":
-            self.listview.expand_node(node.get_parent())
-            self.listview.edit_node(node)
-        elif widget == "":
-            pass
         else:
-            raise Exception("unknown widget '%s'" % widget)
+            self.listview.edit_node(node)
+        
+        #widget = self.get_focused_widget()
+        #
+        #if widget == self.treeview:
+        #    self.treeview.expand_node(node.get_parent())
+        #    self.treeview.edit_node(node)
+        #else:
+        #    self.listview.expand_node(node.get_parent())
+        #    self.listview.edit_node(node)
+
 
 
     def _on_rename_node(self):
         """Callback for renaming a node"""
-        nodes, widget = self.get_selected_nodes()
+        nodes = self.get_selected_nodes()
 
         if len(nodes) == 0:
             return
 
-        if widget == "listview":
-            self.listview.edit_node(nodes[0])
-        elif widget == "treeview":
+        widget = self.get_focused_widget()
+        
+        if widget == self.treeview:
             self.treeview.edit_node(nodes[0])
+        else:
+            self.listview.edit_node(nodes[0])
 
 
-    def goto_node(self, node, direct=True):
+    def goto_node(self, node, direct=False):
         """Move view focus to a particular node"""
 
         if node is None:
@@ -676,33 +729,23 @@ class ThreePaneViewer (Viewer):
     def collapse_node(self, all=False):
         """Collapse the tree beneath the focused node"""
         
-        # TODO: move guts into basetreeview
-
         widget = self.get_focused_widget(self.treeview)
         path, col = widget.get_cursor()
 
         if path:
             if all:
                 # recursively collapse all notes
-                model = widget.get_model()
-                it = model.get_iter(path)
-                def walk(it):
-                    for child in iter_children(model, it):
-                        walk(child)
-                    path2 = model.get_path(it)
-                    widget.collapse_row(path2)
-                walk(it)
+                widget.collapse_all_beneath(path)
             else:
                 widget.collapse_row(path)
 
 
-    def search_nodes(self, text):
-        """Return nodes with titles containing 'text'"""
+    def on_copy_tree(self):
+        """Callback for copy on whole tree"""
+        widget = self._main_window.get_focus()
+        if gobject.signal_lookup("copy-tree-clipboard", widget) != 0:
+            widget.emit("copy-tree-clipboard")
 
-        # TODO: make proper interface
-        nodes = [(nodeid, title) 
-                for nodeid, title in self._notebook._index.search_titles(text)]
-        return nodes
 
 
     #============================================
@@ -717,6 +760,17 @@ class ThreePaneViewer (Viewer):
         """Add a search result"""
         self.listview.append_node(node)
 
+    def end_search_result(self):
+        """End a search result"""
+        
+        # select top result
+        try:
+            self.listview.get_selection().select_path((0,))
+        except:
+            # don't worry if there isn't anything to select
+            pass
+
+
     def viewing_search(self):
         """Returns True if we are currently viewing a search result"""
         return (len(self.treeview.get_selected_nodes()) == 0 and
@@ -736,21 +790,18 @@ class ThreePaneViewer (Viewer):
         
     def goto_editor(self):
         """Switch focus to Editor"""
-        self.editor.get_textview().grab_focus()
-
-    def goto_link(self):
-        """Visit link under cursor"""        
-        self.editor.get_textview().click_iter()
-
+        self.editor.grab_focus()
 
+    
     #===========================================
     # ui
     
     def add_ui(self, window):
         """Add the view's UI to a window"""
-
+        
         assert window == self._main_window
         
+        self._ui_ready = True
         self._action_group = gtk.ActionGroup("Viewer")
         self._uis = []
         add_actions(self._action_group, self._get_actions())
@@ -770,8 +821,8 @@ class ThreePaneViewer (Viewer):
 
 
         # setup editor
-        self.editor_menus.add_ui(window,
-                                 use_minitoolbar=self._app.pref.use_minitoolbar)
+        self.editor.add_ui(window)
+        
         
         # TODO: Try to add accellerator to popup menu
         #menu = viewer.editor.get_textview().get_popup_menu()
@@ -807,14 +858,14 @@ class ThreePaneViewer (Viewer):
 
     def _setup_icon_menu(self):
         """Setup the icon menu"""
-
+        
         iconmenu = IconMenu()
         iconmenu.connect("set-icon",
             lambda w, i: self._app.on_set_icon(
-                i, u"", self.get_selected_nodes()[0]))
+                i, u"", self.get_selected_nodes()))
         iconmenu.new_icon.connect("activate",
             lambda w: self._app.on_new_icon(
-                self.get_selected_nodes()[0], self._notebook, 
+                self.get_selected_nodes(), self._notebook, 
                 self._main_window))
         iconmenu.set_notebook(self._notebook)
 
@@ -827,15 +878,17 @@ class ThreePaneViewer (Viewer):
 
         assert self._main_window == window
 
-        self.editor_menus.remove_ui(self._main_window)
+        self._ui_ready = False
+        self.editor.remove_ui(self._main_window)
 
         for ui in reversed(self._uis):
             self._main_window.get_uimanager().remove_ui(ui)
         self._uis = []
 
+        self._main_window.get_uimanager().ensure_update()
         self._main_window.get_uimanager().remove_action_group(self._action_group)
         self._action_group = None
-        self._main_window.get_uimanager().ensure_update()
+
 
 
     def _get_ui(self):
@@ -889,7 +942,7 @@ class ThreePaneViewer (Viewer):
               <menuitem action="Go to Tree View"/>
               <menuitem action="Go to List View"/>
               <menuitem action="Go to Editor"/>
-              <menuitem action="Go to Link"/>
+              <placeholder name="Editor"/>
             </placeholder>
           </menu>
           <menu action="Tools">
@@ -908,12 +961,19 @@ class ThreePaneViewer (Viewer):
           </placeholder>
         </toolbar>
 
+
         <menubar name="popup_menus">
           <menu action="treeview_popup">
             <menuitem action="New Page"/>
             <menuitem action="New Child Page"/>
             <menuitem action="New Folder"/>
             <menuitem action="Attach File"/>
+            <placeholder name="New"/>
+            <separator/>
+            <menuitem action="Cut"/>
+            <menuitem action="Copy"/>
+            <menuitem action="Copy Tree"/>
+            <menuitem action="Paste"/>
             <separator/>
             <menuitem action="Delete Note"/>
             <menuitem action="Rename Note"/>
@@ -933,6 +993,12 @@ class ThreePaneViewer (Viewer):
             <menuitem action="New Child Page"/>
             <menuitem action="New Folder"/>
             <menuitem action="Attach File"/>
+            <placeholder name="New"/>
+            <separator/>
+            <menuitem action="Cut"/>
+            <menuitem action="Copy"/>
+            <menuitem action="Copy Tree"/>
+            <menuitem action="Paste"/>
             <separator/>
             <menuitem action="Delete Note"/>
             <menuitem action="Rename Note"/>
@@ -957,6 +1023,9 @@ class ThreePaneViewer (Viewer):
             ("treeview_popup", None, "", "", None, lambda w: None),
             ("listview_popup", None, "", "", None, lambda w: None),
 
+            ("Copy Tree", gtk.STOCK_COPY, _("Copy _Tree"),
+             "<control><shift>C", _("Copy entire tree"),
+             lambda w: self.on_copy_tree()),
             
             ("New Page", gtk.STOCK_NEW, _("New _Page"),
              "<control>N", _("Create a new page"),
@@ -971,6 +1040,10 @@ class ThreePaneViewer (Viewer):
              "<control><shift>M", _("Create a new folder"),
              lambda w: self.on_new_dir(),
              "folder-new.png"),
+            
+            ("Attach File", gtk.STOCK_ADD, _("_Attach File..."),
+             "", _("Attach a file to the notebook"),
+             lambda w: self._on_attach_file_menu()),
 
 
             ("Back", gtk.STOCK_GO_BACK, _("_Back"), "", None,
@@ -1023,14 +1096,9 @@ class ThreePaneViewer (Viewer):
             ("Go to Editor", None, _("Go to _Editor"),
              "<control>D", None,
              lambda w: self.goto_editor()),
-            
-            ("Go to Link", None, _("Go to Lin_k"),
-             "<control>space", None,
-             lambda w: self.goto_link()),
-        
+                    
             ("Delete Note", gtk.STOCK_DELETE, _("_Delete"),
-             "", None, 
-             lambda w: self.on_delete_node()),
+             "", None, self.on_delete_node),
 
             ("Rename Note", gtk.STOCK_EDIT, _("_Rename"),
              "", None, 
